import ast
from pcas import keyvalue
import numpy as np
from sklearn.preprocessing import StandardScaler
from sklearn.model_selection import  learning_curve
from sklearn import svm
from sklearn.decomposition import PCA
from sklearn.metrics import accuracy_score
from sklearn.pipeline import Pipeline
from sklearn.utils import shuffle
import statistics as st

target_dim = 4
target_C = 0.001

# Connect to the table
conn = keyvalue.Connection('ml_polydim', address='fanosearch.net:12356')
t = conn.connect_to_table('balanced_quasiperiod')
# Prepare to read the data
template = {'ehrhart': '', 'quasiperiod':0}
this_dim = []
# Read in the data and convert 'ehrhart' to a NumPy array
for d in t.select(template, f'WHERE dimension = {target_dim}'):
    this_dim.append({
        'ehrhart': np.array(ast.literal_eval(d['ehrhart'])),
        'quasiperiod': d['quasiperiod'],
    })
# Clean up
t.close()
conn.close()
# Move to PCA co-ordinates
X = np.array([x['ehrhart'] for x in this_dim])
y = np.array([x['quasiperiod'] for x in this_dim])
pca = PCA()
pca.fit(X)
X_new = pca.fit_transform(X)

# Make pipeline with best hyperparameter values from hyperparameter tuning script
pipe_SVM = Pipeline([
    ('scaler', StandardScaler()),
    ('SVM', svm.LinearSVC(
        random_state = 42,
        C=target_C))
    ])

# Shuffle the data
X_new, y = shuffle(X_new, y, random_state=0)

# Learning curve for different train sizes
train_sizes, train_scores, validation_scores = learning_curve(
    estimator = pipe_SVM,
    X = X_new,
    y = y,
    train_sizes = [0.1,0.2,0.3,0.4,0.5,0.6,0.7,0.8,0.9],
    cv = 5,
    n_jobs=7
    )

# Print and save the results
with open(f'quasiperiod_svm_learning_curve_dim_{target_dim}', 'w') as f:
    print(f'means_train={[st.mean(x) for x in train_scores]}', file=f)
    print(f'means_val={[st.mean(x) for x in validation_scores]}', file=f)
    print(f'sd_train={[st.stdev(x) for x in train_scores]}', file=f)
    print(f'sd_val={[st.stdev(x) for x in validation_scores]}', file=f)
